/*
 * oggCut is a command line tool, to cut a video stream
 *
 * Copyright (C) 2008 Joern Seger
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <iostream>
#include <map>
#include <cstdlib>

#include "fileRepository.h"
#include "streamSerializer.h"
#include "streamMux.h"
#include "oggEncoder.h"
#include "oggStreamEncoder.h"

void printHelpScreen(std::string& progName)
{
  std::cerr << "usage: " << progName << "[options]\n"
            << "Options are:\n"
            << " -i input    : Input file\n"
            << " -o output   : Output file\n"
            << " -s time     : Start time in milliseconds from start\n"
            << "               if no start time is given, 0 is assumed\n"
            << " -e time     : End time in milliseconds\n"
            << "               if no end time or -1 is given, the end of the\n"
            << "               file is assumed as the end\n"
            << " -l length   : Length of the cut area\n";
}

int main(int argc, char* argv[])
{
  int32 startTime(0);
  int32 endTime(-1);
  int32 length(-1);

  std::string inputFile;
  std::string outputFile;
  std::string programName(argv[0]);

  int opt;
  while ((opt = getopt(argc, argv, "hi:o:s:e:d")) != EOF)

  switch (opt) {

    case 'h':
    printHelpScreen(programName);
    exit(-1);

    case 'i':
    inputFile = std::string(optarg);
    break;

    case 'o':
    outputFile = std::string(optarg);
    break;

    case 's':
    startTime = atoi(optarg);
    break;

    case 'e':
    endTime = atoi(optarg);
    break;

    case 'l':
    length = atoi(optarg); // yes, I know the atoi bug
    break;

  }

  /* Handle wrong parameter and parameter combinations */

  if (inputFile.empty() || outputFile.empty()) {
    std::cerr << "Error: please define an input and output file with -i and -o\n\n";
    return(-1);
  }

  if (startTime < 0) {
    std::cerr << "Error: start Time is invalid\n";
    return(-1);
  }

  if (endTime == -1) {
    if (length > 0) {
      endTime = startTime + length;
    }
    else {
      if (startTime == 0) {
        std::cerr << "No need to cut, just use copy!\n";
        return(-1);
      }
    }
  }

  if ((endTime > 0) && (length > 0)) {
    std::cerr << "Warning: end Time and length set, ignoring length\n";
  }

  double startTimeSec(startTime*1.0/1000.0);
  double endTimeSec(endTime*1.0/1000.0);

  StreamSerializer streamSerializer;
  bool foundTheora(false);

  if (!streamSerializer.open(inputFile)) {
    std::cerr << "Error: can not open file <"<<inputFile<<">\n";
    return(-1);
  }

  FileRepository outfile(outputFile, MediaUnit::write);
  StreamMux streamMux(&outfile);

  /* create the headers */
  std::vector<StreamConfig> streamConfigList;
  streamSerializer.getStreamConfig(streamConfigList);

  /* Output some stream information */
  for (uint32 i(0); i<streamConfigList.size(); ++i) {
    std::cerr << streamConfigList[i].parameter->toString();
    if (streamConfigList[i].type == ogg_theora)
      foundTheora = true;
  }

  streamMux.configureStreams(streamConfigList);

  OggPacket packet;
  double time;
  bool startMarker(false);

  /* copy the stream if the packets are within the cut area
   * and the first video keyframe has been found */
  while(1==1) {

    time = streamSerializer.getNextPacket(packet);

    if ((time >= startTimeSec) && (time < endTimeSec)) {

      /* are we within our cut interval and found the first keyframe? */
      if ((!foundTheora) || ((packet.getStreamType() == ogg_theora) &&
          (!(packet.data()[0] & 0x40))))
        startMarker = true;

      /* have we found the real starting position? */
      if (startMarker) {
        // std::cerr << packet.print(2);
        streamMux << packet;
      }

    }

    if (time >= endTimeSec) {
      streamMux.setEndOfStream();
      break;
    }
  }
}
